Imports System.Net
Imports System.IO
Imports CanvasIntegrationSample.ResultsObjects
Imports CanvasIntegrationSample.ResultsParsing

Public Class MainDlg

    Private m_oCurrentSubmission As CanvasSubmission
    Private m_alSubmissions As New ArrayList

    Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click

        btnRefresh.Enabled = False
        Dim strURL As String = APIHelper.GetFormsRequestURL(txtServer.Text.Trim, txtUsername.Text.Trim, txtPassword.Text.Trim)

        If txtFormName.Text.Trim.Length > 0 Then
            strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.FormsRequest.NAME, txtFormName.Text)
        Else
            If txtFormSearch.Text.Trim.Length > 0 Then
                strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.FormsRequest.SEARCH_NAME, txtFormSearch.Text)
            End If
        End If

        Dim strResults As String = ExecuteURL(strURL)
        Dim alForms As ArrayList = ParseAPIResponse(strResults)
        cbForms.Items.Clear()
        cbForms.Text = ""
        For Each oForm As CanvasForm In alForms
            cbForms.Items.Add(oForm)
        Next

        If cbForms.Items.Count > 0 Then
            cbForms.SelectedIndex = 0
        End If

        btnRefresh.Enabled = True

    End Sub
    Private Sub getSubmissions(ByVal page As Int32)
        Dim strURL As String = APIHelper.GetSubmissionsRequestURL(txtServer.Text.Trim, txtUsername.Text.Trim, txtPassword.Text.Trim)

        If txtFormName.Text.Trim.Length > 0 Then
            strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.FORM_NAME, txtFormName.Text)
        Else
            If txtFormSearch.Text.Trim.Length > 0 Then
                strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.SEARCH_NAME, txtFormSearch.Text)
            Else
                If IsNothing(cbForms.SelectedItem) = False Then
                    strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.FORM_ID, CType(cbForms.SelectedItem, CanvasForm).ID.ToString)
                End If
            End If
        End If

        strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.BEGIN_DATE, dtBeginDate.Value.Month.ToString + "/" + dtBeginDate.Value.Day.ToString + "/" + dtBeginDate.Value.Year.ToString)
        strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.END_DATE, dtEndDate.Value.Month.ToString + "/" + dtEndDate.Value.Day.ToString + "/" + dtEndDate.Value.Year.ToString)
        strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.PAGE, page.ToString)

        Dim strResponse As String = ExecuteURL(strURL)
        Dim totalPages = ParseTotalPagesResult(strResponse)
        Dim alSubmissions As ArrayList = ParseAPIResponse(strResponse)
        Dim lviPage As New ListViewItem
        lviPage.Text = "Page " + page.ToString + " of " + totalPages.ToString
        'lvSubmissions.Items.Add(lviPage)
        
        For Each oSubmission As CanvasSubmission In alSubmissions

            Dim lvi As New ListViewItem
            lvi.Text = oSubmission.ResponseID
            lvi.SubItems.Add(oSubmission.Form.ID.ToString)
            lvi.SubItems.Add(oSubmission.Form.Name)
            lvi.SubItems.Add(oSubmission.Form.Version)
            lvi.SubItems.Add(oSubmission.Form.FormStatus)
            lvi.SubItems.Add(oSubmission.UploadDate.ToString)
            lvi.SubItems.Add(oSubmission.Username)
            lvi.SubItems.Add(oSubmission.Detail)
            lvSubmissions.Items.Add(lvi)
            m_alSubmissions.Add(oSubmission)
        Next
        If page < totalPages Then
            getSubmissions(page + 1)
        End If
    End Sub

    Private Sub btnRefreshSubmissions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefreshSubmissions.Click

        btnRefreshSubmissions.Enabled = False
        lvSubmissions.Items.Clear()
        m_alSubmissions.Clear()
        getSubmissions(1)


        If lvSubmissions.Items.Count > 0 Then
            lvSubmissions.Items(0).Selected = True
        End If

        btnRefreshSubmissions.Enabled = True
    End Sub

    Private Sub btnViewImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnViewImage.Click

        btnViewImage.Enabled = False
        Dim strURL As String = APIHelper.GetImagesRequestURL(txtServer.Text.Trim, txtUsername.Text.Trim, txtPassword.Text.Trim)

        If txtImageID.Text.Trim.Length > 0 Then
            strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.ImagesRequest.IMAGE_ID, txtImageID.Text.Trim)
        End If

        Dim oResponseObject As Object = ExecuteURL(strURL, True)

        ParseImageResponse(oResponseObject)

        btnViewImage.Enabled = True


    End Sub

    Private Sub lvSubmissions_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvSubmissions.SelectedIndexChanged
        txtDetails.Text = ""
        m_oCurrentSubmission = Nothing
        btnViewTree.Enabled = False

        If lvSubmissions.SelectedIndices.Count > 0 Then
            If lvSubmissions.Items(lvSubmissions.SelectedIndices(0)).SubItems.Count > 1 Then
                txtDetails.Text = lvSubmissions.Items(lvSubmissions.SelectedIndices(0)).SubItems(7).Text
                m_oCurrentSubmission = m_alSubmissions(lvSubmissions.SelectedIndices(0))
                btnViewTree.Enabled = True
            End If
        End If
    End Sub

    Private Sub btnClearForms_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClearForms.Click
        cbForms.Items.Clear()
        cbForms.Text = ""
    End Sub

    Private Sub MainDlg_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        dtBeginDate.Value = DateAdd(DateInterval.Month, -1, Now)
        dtEndDate.Value = Now
    End Sub

    Private Sub btnViewTree_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnViewTree.Click
        If IsNothing(m_oCurrentSubmission) = False Then
            Dim oResDlg As New ResultsDlg(m_oCurrentSubmission)
            oResDlg.ShowDialog()
            oResDlg.Close()
        End If
    End Sub

    Private Sub txtImageID_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtImageID.TextChanged
        btnViewImage.Enabled = txtImageID.Text.Trim.Length > 0
    End Sub

    Private Function ExecuteURL(ByVal URL As String, Optional ByVal BinaryRead As Boolean = False, Optional ByVal postData As String = "", Optional ByVal Token As String = "") As Object

        Dim retObject As Object

        Try

            Dim request As HttpWebRequest = CType(WebRequest.Create(URL), HttpWebRequest)
            request.Timeout = 100000000

            request.Method = "POST"
            If Token.Length > 0 Then
                request.Credentials = New NetworkCredential(Token, "")
            End If

            setProxySettings(request)

            Dim byteArray As Byte() = System.Text.Encoding.UTF8.GetBytes(postData)
            Dim postStream As Stream = request.GetRequestStream
            postStream.Write(byteArray, 0, byteArray.Length)
            postStream.Close()

            Dim response As HttpWebResponse


            Try

                response = CType(request.GetResponse(), HttpWebResponse)

                Dim streamResponse As Stream = response.GetResponseStream()

                Select Case (response.StatusCode)
                    Case System.Net.HttpStatusCode.OK
                        'read the content from the response

                        If BinaryRead Then

                            Dim aByte() As Byte
                            aByte = New Byte(4096) {}
                            Dim oMS As New IO.MemoryStream
                            Dim count As Int32 = -1
                            While count <> 0
                                count = streamResponse.Read(aByte, 0, aByte.Length)
                                oMS.Write(aByte, 0, count)
                            End While

                            retObject = oMS

                        Else

                            Dim streamRead As StreamReader = New StreamReader(streamResponse)
                            retObject = streamRead.ReadToEnd
                            streamRead.Close()

                        End If

                    Case System.Net.HttpStatusCode.Redirect, System.Net.HttpStatusCode.MovedPermanently
                        MessageBox.Show(String.Format( _
                            "Unable to read response content.  URL has moved. StatusCode={0}.", _
                            response.StatusCode), "Web Exception")
                    Case System.Net.HttpStatusCode.NotFound
                        MessageBox.Show(String.Format( _
                            "Unable to read response content. URL not found. StatusCode={0}.", _
                            response.StatusCode), "Web Exception")
                    Case Else
                        MessageBox.Show(String.Format( _
                            "Unable to read response content. StatusCode={0}.", _
                            response.StatusCode), "Web Exception")
                End Select
            Catch we As System.Net.WebException
                'If (we.Status = Net.WebExceptionStatus.Timeout) Then
                '    Return False
                'End If
                MessageBox.Show(we.Message)
                MessageBox.Show("Unable to contact server.", "Communication Error")
            Finally
                If (Not IsNothing(response)) Then
                    response.Close()
                End If
            End Try

        Catch ex As Exception
            MessageBox.Show(ex.Message)
            MessageBox.Show("Unable to contact server.", "Communication Error")
        End Try

        Return retObject

    End Function

    Private Sub setProxySettings(ByRef request As HttpWebRequest)

        If cbProxyEnabled.checked Then

            request.Proxy = WebRequest.DefaultWebProxy
            Dim username As String = txtProxyUser.Text
            Dim password As String = txtProxyPassword.Text
            ' Create a NetworkCredential object and associate it with the Proxy property of request object.
            request.Proxy.Credentials = New NetworkCredential(username, password)

        End If

    End Sub

    Private Sub cbProxyEnabled_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbProxyEnabled.CheckedChanged
        txtProxyUser.Enabled = cbProxyEnabled.Checked
        txtProxyPassword.Enabled = cbProxyEnabled.Checked
        lblProxyUser.Enabled = cbProxyEnabled.Checked
        lblProxyPassword.Enabled = cbProxyEnabled.Checked

    End Sub

    Private Sub btnGetCSV_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGetCSV.Click

        Dim strURL As String = APIHelper.GetCSVRequestURL(txtServer.Text.Trim, txtUsername.Text.Trim, txtPassword.Text.Trim)

        If txtFormName.Text.Trim.Length > 0 Then
            strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.FORM_NAME, txtFormName.Text)
        Else
            If txtFormSearch.Text.Trim.Length > 0 Then
                strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.SEARCH_NAME, txtFormSearch.Text)
            Else
                If IsNothing(cbForms.SelectedItem) = False Then
                    strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.FORM_ID, CType(cbForms.SelectedItem, CanvasForm).ID.ToString)
                End If
            End If
        End If

        strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.BEGIN_DATE, dtBeginDate.Value.Month.ToString + "/" + dtBeginDate.Value.Day.ToString + "/" + dtBeginDate.Value.Year.ToString)
        strURL += APIHelper.GetQuerySringParameter(APIHelper.Constants.Parameters.SubmissionRequest.END_DATE, dtEndDate.Value.Month.ToString + "/" + dtEndDate.Value.Day.ToString + "/" + dtEndDate.Value.Year.ToString)
        
        Dim strResponse As String = ExecuteURL(strURL)

        MessageBox.Show(strResponse)


    End Sub
End Class
