Imports CanvasIntegrationSample.ResultsObjects
Imports CanvasIntegrationSample.ResultsConstants

Public Class ResultsParsing

    Public Shared Function ParseAPIResponse(ByVal SubmissionsResp As String) As ArrayList

        Dim oDoc As New Xml.XmlDocument
        oDoc.LoadXml(SubmissionsResp)
        Dim oResultsNode As Xml.XmlNode

        For Each oNode As Xml.XmlNode In oDoc.ChildNodes
            If oNode.Name = Constants.CanvasResultNode.NAME Then
                oResultsNode = oNode
                Exit For
            End If
        Next

        Return ResultsParsing.ParseCanvasResult(oResultsNode)

    End Function

    Public Shared Function ParseImageResponse(ByVal ImageResp As System.IO.MemoryStream)

        Try
            Dim oImage As Image = Image.FromStream(CType(ImageResp, System.IO.Stream))
            Dim oImageDlg As New ImageDlg(oImage)
            oImageDlg.Width = oImage.Width
            oImageDlg.Height = oImage.Height
            oImageDlg.ShowDialog()
            oImageDlg.Dispose()

            oImage.Save("C:\Users\Matthew\Downloads\test_" + MainDlg.txtImageID.Text + ".jpg")

        Catch ex As Exception
            Dim aByte As Byte()
            aByte = ImageResp.ToArray
            Dim strResp As String = ""
            Dim i As Int32
            For i = 0 To aByte.Length - 1
                strResp += Chr(aByte(i))
            Next
            ParseAPIResponse(strResp)
        End Try

        Return Nothing

    End Function

    Public Shared Function ParseCanvasResult(ByVal ResultNode As Xml.XmlNode) As ArrayList

        Dim alResults As New ArrayList

        For Each oResultNode As Xml.XmlNode In ResultNode.ChildNodes

            If oResultNode.Name = Constants.CanvasResultNode.SubNodes.SUBMISSIONS Then

                Return ParseSubmissions(oResultNode)

            End If

            If oResultNode.Name = Constants.CanvasResultNode.SubNodes.FORMS Then

                Return ParseForms(oResultNode)

            End If

            If oResultNode.Name = Constants.CanvasResultNode.SubNodes.ERROR_NODE Then

                ParseError(oResultNode)

            End If

        Next

        Return alResults

    End Function
    Public Shared Function ParseTotalPagesResult(ByVal SubmissionsResp As String) As Int32
        Dim oDoc As New Xml.XmlDocument
        oDoc.LoadXml(SubmissionsResp)

        For Each oNode As Xml.XmlNode In oDoc.ChildNodes

            If oNode.Name = Constants.CanvasResultNode.NAME Then

                For Each oResultNode As Xml.XmlNode In oNode.ChildNodes

                    If oResultNode.Name = Constants.CanvasResultNode.SubNodes.TOTAL_PAGES Then

                        Return oResultNode.InnerText

                    End If

                Next
                Exit For
            End If
        Next

        Return 0

    End Function

    Public Shared Function ParseForms(ByVal FormsNode As Xml.XmlNode) As ArrayList

        Dim alForms As New ArrayList

        For Each oChild As Xml.XmlNode In FormsNode.ChildNodes
            alForms.Add(ParseForm(oChild))
        Next
        Return alForms

    End Function

    Public Shared Function ParseForm(ByVal FormNode As Xml.XmlNode) As CanvasForm

        Dim oForm As New CanvasForm(0, "")
        For Each oChilds As Xml.XmlNode In FormNode.ChildNodes
            oForm.ID = FormNode.Attributes(Constants.FormNode.Attributes.ID).Value
            If oChilds.Name = Constants.FormNode.SubNodes.NAME Then
                oForm.Name = oChilds.InnerText
            ElseIf oChilds.Name = Constants.FormNode.SubNodes.STATUS Then
                oForm.FormStatus = oChilds.InnerText
            ElseIf oChilds.Name = Constants.FormNode.SubNodes.VERSION Then
                oForm.Version = oChilds.InnerText
            End If
        Next

        Return oForm

    End Function

    Public Shared Function ParseSubmissions(ByVal SubmissionsNode As Xml.XmlNode) As ArrayList

        Dim alSubs As ArrayList = New ArrayList

        For Each oSubNode As Xml.XmlNode In SubmissionsNode.ChildNodes

            If oSubNode.Name = Constants.SubmissionsNode.SubNodes.SUBMISSION Then

                alSubs.Add(ParseSubmission(oSubNode))

            End If

        Next

        Return alSubs

    End Function

    Public Shared Function ParseSubmission(ByVal SubmissionNode As Xml.XmlNode) As CanvasSubmission

        Dim oForm As New CanvasForm
        Dim oSub As New CanvasSubmission


        For Each oChild As Xml.XmlNode In SubmissionNode.ChildNodes

            If oChild.Name = Constants.SubmissionNode.SubNodes.DATE_NODE Then
                oSub.UploadDate = oChild.InnerText
            End If

            If oChild.Name = Constants.SubmissionNode.SubNodes.USER_NAME Then
                oSub.Username = oChild.InnerText
            End If

            If oChild.Name = Constants.SubmissionNode.SubNodes.RESPONSE_ID Then
                oSub.ResponseID = oChild.InnerText
            End If

            If oChild.Name = Constants.SubmissionNode.SubNodes.SECTIONS Then
                oSub.Detail = oChild.InnerXml
                oSub.Sections = ParseSections(oChild)
            End If

            If oChild.Name = Constants.SubmissionNode.SubNodes.FORM Then

                oForm = ParseForm(oChild)

            End If

        Next

        oSub.Form = oForm

        Return oSub

    End Function

    Public Shared Function ParseSections(ByVal SectionsNode As Xml.XmlNode) As ArrayList

        Dim alSections As New ArrayList

        For Each SectionNode As Xml.XmlNode In SectionsNode.ChildNodes
            alSections.Add(ParseSection(SectionNode))
        Next

        Return alSections


    End Function

    Public Shared Function ParseSection(ByVal SectionNode As Xml.XmlNode) As CanvasSection

        Dim oSection As New CanvasSection

        For Each oChild As Xml.XmlNode In SectionNode
            If oChild.Name = Constants.SectionNode.SubNodes.Name Then
                oSection.Name = oChild.InnerText
            End If

            If oChild.Name = Constants.SectionNode.SubNodes.SCREENS Then
                oSection.Screens = ParseScreens(oChild)
            End If

        Next

        Return oSection

    End Function

    Public Shared Function ParseScreens(ByVal ScreensNode As Xml.XmlNode) As ArrayList

        Dim alScreens As New ArrayList
        For Each oChild As Xml.XmlNode In ScreensNode

            If oChild.Name = Constants.ScreensNode.SubNodes.SCREEN Then
                alScreens.Add(ParseScreen(oChild))
            End If

        Next

        Return alScreens

    End Function

    Public Shared Function ParseScreen(ByVal ScreenNode As Xml.XmlNode) As CanvasScreen

        Dim oScreen As New CanvasScreen

        For Each oChild As Xml.XmlNode In ScreenNode
            If oChild.Name = Constants.ScreenNode.SubNodes.NAME Then
                oScreen.Name = oChild.InnerText
            End If

            If oChild.Name = Constants.ScreenNode.SubNodes.RESPONSE_GROUPS Then
                oScreen.ResponseGroups = ParseResponseGroups(oChild)
            End If

            If oChild.Name = Constants.ScreenNode.SubNodes.RESPONSES Then
                oScreen.Responses = ParseResponses(oChild)
            End If
        Next

        Return oScreen

    End Function

    Public Shared Function ParseResponses(ByVal ResponsesNode As Xml.XmlNode) As ArrayList

        Dim alResponses As New ArrayList
        For Each oChild As Xml.XmlNode In ResponsesNode

            If oChild.Name = Constants.ResponsesNode.SubNodes.RESPONSE Then
                alResponses.Add(ParseResponse(oChild))
            End If

        Next

        Return alResponses

    End Function

    Public Shared Function ParseResponse(ByVal ResponseNode As Xml.XmlNode) As CanvasResponse

        Dim oResponse As New CanvasResponse

        For Each oChild As Xml.XmlNode In ResponseNode
            If oChild.Name = Constants.ResponseNode.SubNodes.LABEL Then
                oResponse.Label = oChild.InnerText
            End If

            If oChild.Name = Constants.ResponseNode.SubNodes.VALUE Then
                oResponse.Value = oChild.InnerText
            End If

            If oChild.Name = Constants.ResponseNode.SubNodes.TYPE Then
                oResponse.Type = oChild.InnerText
            End If
        Next

        Return oResponse

    End Function

    Public Shared Function ParseResponseGroups(ByVal ResponseGroupsNode As Xml.XmlNode) As ArrayList

        Dim alResponseGroups As New ArrayList
        For Each oChild As Xml.XmlNode In ResponseGroupsNode

            If oChild.Name = Constants.ResponseGroupsNode.SubNodes.RESPONSE_GROUP Then
                alResponseGroups.Add(ParseResponseGroup(oChild))
            End If

        Next

        Return alResponseGroups

    End Function

    Public Shared Function ParseResponseGroup(ByVal ResponseGroupNode As Xml.XmlNode) As CanvasResponseGroup

        Dim oGroup As New CanvasResponseGroup
        For Each oChild As Xml.XmlNode In ResponseGroupNode

            If oChild.Name = Constants.ResponseGroupNode.SubNodes.RESPONSE Then
                oGroup.KeyReponse = ParseResponse(oChild)
            End If

            If oChild.Name = Constants.ResponseGroupNode.SubNodes.SECTION Then
                oGroup.Section = ParseSection(oChild)
            End If

        Next

        Return oGroup

    End Function

    Public Shared Function ParseError(ByVal ErrorNode As Xml.XmlNode) As String

        Dim strCode As String = ""
        Dim strDescription As String = ""
        For Each oChild As Xml.XmlNode In ErrorNode

            If oChild.Name = Constants.ErrorNode.SubNodes.ERROR_CODE Then
                strCode = oChild.InnerText
            End If

            If oChild.Name = Constants.ErrorNode.SubNodes.DESCRIPTION Then
                strDescription = oChild.InnerText
            End If

        Next

        MessageBox.Show(strDescription + " (" + strCode + ")", "An Error Occurred.")

        Return strDescription

    End Function

End Class
