Imports System.Net
Imports System.IO

Public Class ImportDlg

    Public Shared Function NewAtribute(ByVal XMLDom As Xml.XmlDocument, ByVal Name As String, ByVal Value As String) As Xml.XmlAttribute
        Dim oAttribute As Xml.XmlAttribute = XMLDom.CreateNode(Xml.XmlNodeType.Attribute, Name, "")
        oAttribute.Value = Value
        Return oAttribute
    End Function

    Private Sub btnImport_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnImport.Click
        SaveSettings()
        Me.Enabled = False

        Try

            Dim oDoc As New Xml.XmlDocument()

            Dim myXMLString As String = LoadCSV(txtFileName.Text)

            Dim strURL As String = APIHelper.GetReferenceDataImportURL(txtServer.Text.Trim, txtUsername.Text.Trim, txtPassword.Text.Trim)

            Dim strResults As String = ExecuteURL(strURL, myXMLString)

            Dim oResults As New Xml.XmlDocument
            oResults.LoadXml(strResults)

            Dim oErrorNode As Xml.XmlNode = oResults.SelectSingleNode("CanvasResult/Error")

            If Not IsNothing(oErrorNode) Then
                Dim strErrorCode As String = oErrorNode.SelectSingleNode("ErrorCode").InnerText
                Dim strErrorDesc As String = oErrorNode.SelectSingleNode("Description").InnerText
                If String.Compare("0", strErrorCode, True) <> 0 Then
                    MessageBox.Show("Error Occurred: " + strErrorCode + " - " + strErrorDesc)
                End If
            End If


        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try

        Me.Enabled = True

    End Sub

    Private Function LoadCSV(ByVal fileName As String) As String

        '        <List Name="Properties" UserGroupColumn="Manager">
        '<Columns><c>Manager</c><c>Address1</c></Columns>
        '<Rows>
        '<r><v>Josh Daigle</v><v>4383 Hecktown Road FOCL FOCC</v></r>
        '        <r><v>Barbara Hammer</v><v>30 MAIN STREET APT- A</v></r>
        '</Rows>
        '</List>

        Dim strXML As String = "<List Name=""" + txtDatasetName.Text + """>"

        strXML += "<Columns>"
        Dim dt As DataTable = GetDataTable(fileName)
        For Each dc As DataColumn In dt.Columns
            strXML += "<c>"
            strXML += dc.ColumnName
            strXML += "</c>"
        Next
        strXML += "</Columns>"

        strXML += "<Rows>"
        For Each dr As DataRow In dt.Rows
            strXML += "<r>"
            For Each value As String In dr.ItemArray
                strXML += "<v>"
                strXML += value
                strXML += "</v>"
            Next
            strXML += "</r>"
        Next
        strXML += "</Rows>"

        strXML += "</List>"

        Return strXML


    End Function

    Private Function GetDataTable(ByVal fielName As String) As DataTable

        Dim SR As StreamReader = New StreamReader(fielName)
        Dim line As String = SR.ReadLine()
        Dim strArray As String() = line.Split(","c)
        Dim dt As DataTable = New DataTable()
        Dim row As DataRow

        For Each s As String In strArray
            dt.Columns.Add(New DataColumn())
        Next

        Do
            line = SR.ReadLine
            If Not line = String.Empty Then
                row = dt.NewRow()
                row.ItemArray = line.Split(","c)
                dt.Rows.Add(row)
            Else
                Exit Do
            End If
        Loop

        Return dt

    End Function



    Private Function ExecuteURL(ByVal URL As String, ByVal postData As String, Optional ByVal BinaryRead As Boolean = False) As Object

        URL = URL.Replace(" ", "%20")
        Dim retObject As Object

        Try

            Dim request As HttpWebRequest = CType(WebRequest.Create(URL), HttpWebRequest)

            Dim byteArray As Byte() = System.Text.Encoding.UTF8.GetBytes(postData)

            request.ContentType = "application/xml"
            request.ContentLength = byteArray.Length
            'request.Timeout = 4000
            request.Method = "POST"

            Dim postStream As Stream = request.GetRequestStream
            postStream.Write(byteArray, 0, byteArray.Length)
            postStream.Close()

            Dim response As HttpWebResponse

            Try

                response = CType(request.GetResponse(), HttpWebResponse)
                Dim streamResponse As Stream = response.GetResponseStream()

                Select Case (response.StatusCode)
                    Case System.Net.HttpStatusCode.OK
                        'read the content from the response

                        If BinaryRead Then

                            Dim aByte() As Byte
                            aByte = New Byte(4096) {}
                            Dim oMS As New IO.MemoryStream
                            Dim count As Int32 = -1
                            While count <> 0
                                count = streamResponse.Read(aByte, 0, aByte.Length)
                                oMS.Write(aByte, 0, count)
                            End While

                            retObject = oMS

                        Else

                            Dim streamRead As StreamReader = New StreamReader(streamResponse)
                            retObject = streamRead.ReadToEnd
                            streamRead.Close()

                        End If

                    Case System.Net.HttpStatusCode.Redirect, System.Net.HttpStatusCode.MovedPermanently
                        MessageBox.Show(String.Format( _
                            "Unable to read response content.  URL has moved. StatusCode={0}.", _
                            response.StatusCode), "Web Exception")
                    Case System.Net.HttpStatusCode.NotFound
                        MessageBox.Show(String.Format( _
                            "Unable to read response content. URL not found. StatusCode={0}.", _
                            response.StatusCode), "Web Exception")
                    Case Else
                        MessageBox.Show(String.Format( _
                            "Unable to read response content. StatusCode={0}.", _
                            response.StatusCode), "Web Exception")
                End Select
            Catch we As System.Net.WebException
                'If (we.Status = Net.WebExceptionStatus.Timeout) Then
                '    Return False
                'End If
                MessageBox.Show(we.Message)
                MessageBox.Show("Unable to contact server.", "Communication Error")
            Finally
                If (Not IsNothing(response)) Then
                    response.Close()
                End If
            End Try

        Catch ex As Exception
            MessageBox.Show(ex.Message)
            MessageBox.Show("Unable to contact server.", "Communication Error")
        End Try

        Return retObject

    End Function

    Private Sub ImportDlg_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        SaveSettings()
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        LoadSettings()
    End Sub

    Private Sub SaveSettings()

        Dim oDoc As New Xml.XmlDocument()

        Dim oSettingsNode As Xml.XmlNode = oDoc.CreateNode(Xml.XmlNodeType.Element, "", "Settings", "")
        oSettingsNode.Attributes.Append(NewAtribute(oDoc, "ServerURL", Me.txtServer.Text))
        oSettingsNode.Attributes.Append(NewAtribute(oDoc, "Username", Me.txtUsername.Text))
        oSettingsNode.Attributes.Append(NewAtribute(oDoc, "Password", Me.txtPassword.Text))
        oSettingsNode.Attributes.Append(NewAtribute(oDoc, "ImportType", "0"))
        oSettingsNode.Attributes.Append(NewAtribute(oDoc, "FileName", Me.txtFileName.Text))
        oSettingsNode.Attributes.Append(NewAtribute(oDoc, "DatasetName", Me.txtDatasetName.Text))

        oDoc.AppendChild(oSettingsNode)

        oDoc.Save("Settings.xml")

    End Sub

    Private Sub LoadSettings()

        If File.Exists("Settings.xml") Then
            Dim oDoc As New Xml.XmlDocument()
            oDoc.Load("Settings.xml")

            Dim oSettingsNode As Xml.XmlNode = oDoc.SelectSingleNode("Settings")

            Try

                txtServer.Text = oSettingsNode.Attributes("ServerURL").Value
                txtUsername.Text = oSettingsNode.Attributes("Username").Value
                txtPassword.Text = oSettingsNode.Attributes("Password").Value
                txtFileName.Text = oSettingsNode.Attributes("FileName").Value
                txtDatasetName.Text = oSettingsNode.Attributes("DatasetName").Value
            Catch Ex As Exception
            End Try
        End If

    End Sub

    Private Sub btnSelect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSelect.Click
        If OpenFileDialog1.ShowDialog = vbOK Then
            txtFileName.Text = OpenFileDialog1.FileName
        End If

    End Sub
End Class
